﻿using Verse;

namespace NzS_EternalYouthHediff
{
    [StaticConstructorOnStartup]
    internal class HediffComp_EternalYouthHediff : HediffComp
    {
        // this is how often to tick action is called
        private uint tickDelay = 1200;

        public string labelCap
        {
            get
            {
                return base.Def.LabelCap;
            }
        }

        public string label
        {
            get
            {
                return base.Def.label;
            }
        }

        public override void CompPostTick(ref float severityAdjustment)
        {
            base.CompPostTick(ref severityAdjustment);

            if (Find.TickManager.TicksGame % tickDelay == 0)
            {
                TickAction();
            }
        }

        public void TickAction()
        {
            this.Pawn.ageTracker.AgeBiologicalTicks -= tickDelay;
        }
    }

    /// <summary>
    /// A pawn will age normally between 13 and 20
    /// But they can't go above, or below
    /// If they start below 13, then they'll age normally till they're 13
    /// </summary>
    [StaticConstructorOnStartup]
    internal class HediffComp_EternalTeen : HediffComp
    {
        // this is how often to tick action is called
        private uint tickDelay = 3600;
        private uint ticksPerYear = 3600000;
        private uint maxBiologicalYears = 20;
        private uint minBiologicalYears = 13;

        private bool initialized = false;
        private long initialAgeTick = 0;
        private long actualAgeTick = 0;

        public string labelCap
        {
            get
            {
                return base.Def.LabelCap;
            }
        }

        public string label
        {
            get
            {
                return base.Def.label;
            }
        }

        public override void CompPostTick(ref float severityAdjustment)
        {
            base.CompPostTick(ref severityAdjustment);

            long currentAgeTick = this.Pawn.ageTracker.AgeBiologicalTicks;

            if (!initialized)
            {
                initialAgeTick = currentAgeTick;
                actualAgeTick = currentAgeTick;

                initialized = true;
            }

            actualAgeTick++;

            long maxTicks = ticksPerYear * maxBiologicalYears - tickDelay;
            long minTicks = ticksPerYear * minBiologicalYears + tickDelay;

            // if age exceeds max, it gets set back to max
            if (currentAgeTick > maxTicks)
            {
                this.Pawn.ageTracker.AgeBiologicalTicks = maxTicks;
            }
            // if age is lower than min, it gets set to min (mainly for ARoM reverse age)
            else if (currentAgeTick < minTicks && actualAgeTick > minTicks)
            {
                this.Pawn.ageTracker.AgeBiologicalTicks = minTicks;
            }

        }

        public void TickAction()
        {
            this.Pawn.ageTracker.AgeBiologicalTicks = ticksPerYear * maxBiologicalYears;
        }
    }
}
